package build

import (
	"path/filepath"
	"strings"
)

// Build describes a compiler build.
type Build struct {
	Files           []string
	WriteExecutable bool
}

// New creates a new build.
func New(files ...string) *Build {
	return &Build{
		Files:           files,
		WriteExecutable: true,
	}
}

// Run parses the input files and generates an executable file.
func (build *Build) Run() error {
	functions, errors := Scan(build.Files)
	allFunctions, err := Compile(functions, errors)

	if err != nil {
		return err
	}

	if !build.WriteExecutable {
		return nil
	}

	path := build.Executable()
	code, data := Finalize(allFunctions)
	return Write(path, code, data)
}

// Executable returns the path to the executable.
func (build *Build) Executable() string {
	directory, _ := filepath.Abs(build.Files[0])

	if strings.HasSuffix(directory, ".q") {
		directory = filepath.Dir(directory)
	}

	return filepath.Join(directory, filepath.Base(directory))
}
